library nui_form;

import 'package:flutter/cupertino.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'nuif_form_component.dart';

export 'package:nui_form/nuif_form_component.dart';
export 'package:nui_form/component/nuif_text.dart';
export 'package:nui_form/component/nuif_dropdown.dart';
export 'package:nui_form/component/nuif_radio.dart';
export 'package:nui_form/component/nuif_button.dart';
export 'package:nui_form/component/nuif_checkbox.dart';
export 'package:nui_form/component/nuif_checkgroup.dart';
export 'package:nui_form/component/nuif_column.dart';
export 'package:nui_form/component/nuif_row.dart';
export 'package:nui_form/component/nuif_list.dart';
export 'package:nui_form/component/nuif_custom.dart';
export 'package:nui_form/model/nuif_models.dart';

typedef AllTextChangeListener = void Function(String inpId, String value);
typedef AllItemSelectListener = void Function(String inpId, NUIFormSelection selection, bool selected);
typedef AllButtonClickListener = void Function(String inpId, ClickType clickType);
typedef FormTextChangeListener = void Function(String inpId, String value, NUIForm form);
typedef FormButtonClickListener = void Function(String inpId, ClickType clickType, NUIForm form);
typedef FormItemSelectListener = void Function(String inpId, NUIFormSelection selection, bool selected, NUIForm form);
typedef FormInitiator = NUIForm Function(FormTextChangeListener allTextChangeListener, FormButtonClickListener allButtonClickListener, FormItemSelectListener allItemSelectListener);
typedef UpdateFormItem<T extends NUIFormComponent> = T Function(T item);
typedef StateGetter = NUIFState Function();

enum ClickType{
  TAP,
  LONG_TAP,
  DOUBLE_TAP,
  CHECKED,
  UNCHECKED
}

class NUIForm {
  NUIForm({@required this.item, @required this.allButtonClickListener, @required this.allTextChangeListener, @required this.allItemSelectListener});
  NUIFormComponent item;
  final FormTextChangeListener allTextChangeListener;
  final FormButtonClickListener allButtonClickListener;
  final FormItemSelectListener allItemSelectListener;

  Map<String, NUIFormValue> extractValues(){
    Map<String, NUIFormValue> values = Map();
    item.extractValue(values);
    return values;
  }

  bool validate(bool quiet){
    return item.validate(quiet);
  }

  T updateItem<T extends NUIFormComponent>(String inpId, UpdateFormItem<T> updateTask){
    final target = item.findItemById(inpId);
    if(target != null){
      final updatedItem = updateTask(target);
      if(updatedItem?.stateGetter != null){
        updatedItem?.stateGetter().update(updatedItem);
      }
    }
    return target;
  }

}

class NUIFormContent extends StatefulWidget {
  NUIFormContent({this.form, Key key}): super(key: key);
  final NUIForm form;
  @override
  _NUIFormContentState createState() => _NUIFormContentState();
}

class _NUIFormContentState extends State<NUIFormContent> {

  @override
  Widget build(BuildContext context) {
    return Container(
      key: widget.key,
      child: widget.form.item.toWidget(
        allButtonClickListener: (inpId, clickType){
          if(widget.form.allButtonClickListener != null) {
            widget.form.allButtonClickListener(inpId, clickType, widget.form);
          }
        },
        allTextChangeListener: (inpId, value){
          if(widget.form.allTextChangeListener != null) {
            widget.form.allTextChangeListener(inpId, value, widget.form);
          }
        },
        allItemSelectListener: (inpId, selection, selected){
          if(widget.form.allItemSelectListener != null) {
            widget.form.allItemSelectListener(inpId, selection, selected, widget.form);
          }
        },
        key: widget.key)
    );
  }
}


